;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH BULLETS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "bullet.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "player.h.s"

.globl hero_width

pos_x_init: .db 00
hero_width: .db 0x00


;;==============================================================
;; Initialize bullet position
;; DESTROYS: IX
;;==============================================================
bullet_init:

   

    call    player_getPlayer
    ld      a, de_x(ix)
    ld      (pos_x_init), a

    

    ret

;;=============================================
;; Update bullet position
;; INPUT: IX -> Ptr to bullet array
;; DESTROYS: A, HL
;;=============================================
bullet_update:

    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;; Update the position
    ld  h, e_up_h(ix)
    ld  l, e_up_l(ix)       
    jp  (hl)


;;=============================================
;; Draw the bullets
;; INPUT: IX -> Ptr to bullet array
;; DESTROYS: A
;;=============================================
bullet_draw:
    
    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;;Draws on the screen
    jp    ren_drawEntity


;;=============================================
;; Destroy the bullets
;; INPUT: IX -> Ptr to bullet array
;;=============================================
bullet_erase:

    ld      de_x(ix), #0xFF
    ld      de_y(ix), #0xFF

    ret

;;====================================================
;; Update bullet position based on the hero's position
;; INPUT: IX -> Ptr to bullet array
;; DESTROYS: A, IY
;;====================================================
bullet_posInicialH:

    call   player_getPlayer
    ld     iy, #disparo_data

    ;; Check right side
    ld  a, (hero_width)
    cp  #0x01
    jr nz, pasa
        ld      a, de_x(ix)
        ld      (pos_x_init), a

pasa:
    ;; Check left side
    ld  a, (hero_width)
    cp  #0x00
    jr nz, pasa2
        ld      a, de_x(ix)
        ld      (pos_x_init), a

pasa2:
    ;; We update data
    ld      a, (pos_x_init)
    ld      b, de_y(ix)

    ld      de_x(iy), a
    ld      de_y(iy), b


    
     
    ret

;;====================================================
;; Update bullet position based on enemy position
;; DESTROYS: A, IY, IX
;;====================================================
bullet_posInicialE:

    ld     iy, #enemy_data

    call    bullet_getBulletE

    ld      a, e_vx(ix)
    add #2
    jr nz, der 

    ld      a, de_x(iy)
    sub #2
    
    jp colocar

der:
    ld a,de_x(iy)
    add de_w(iy)
    sub #1

colocar:
    ld      b, de_y(iy)
    ld      de_x(ix), a
    ld      (pos_x_init), a
    ld      de_y(ix), b

    ret

;;====================================================
;; Check for a bullet and initialize its position
;; INPUT: IX -> Ptr to bullet array
;; DESTROYS: A, IX
;;====================================================
bullet_check:

    ;; Check if there is another bullet created
    ld 	a, de_x(ix)
    cp 	#0xFF
    ret 	nz

    call    player_getPlayer
    ld      a, de_x(ix)
    ld      (pos_x_init), a

    ;; If it is created, we update the position of the bullet
    call    bullet_posInicialH

    ret

;;============================================================
;; Eliminates the bullet in case of reaching one of its limits
;; INPUT: IX -> Ptr to bullet array
;; DESTROYS: A
;;============================================================
bullet_limits:

    ld a, (pos_x_init)
    add #0x12
    cp de_x(ix)
    jp m, limits

    ld a, (pos_x_init)
    sub #0x12
    ld b, a
    ld a, de_x(ix)
    cp b
    jp m, limits

    ret

    limits:

        call bullet_erase

    ret

;;====================================================
;; Get the hero's bullet
;; DESTROYS: IX
;;====================================================
bullet_getBullet:

    ld	ix, #disparo_data

    ret


;;====================================================
;; Get the enemy bullet
;; DESTROYS: IX
;;====================================================
bullet_getBulletE:

    ld  ix, #disparoE_data

    ret

