;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH COLLISIONS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "main.h.s"
.include "entity.h.s"
.include "collision.h.s"
.include "bullet.h.s"
.include "enemy.h.s"
.include "player.h.s"
.include "menu.h.s"
.include "obstacle.h.s"
.include "map.h.s"
.include "level.h.s"

limit: .db #00

DefineWorld world_data, 0x08, 0x01, 0x50, 0x30

;;====================================================
;; Check all game collisions
;; DESTROYS: B, IY, IX
;;====================================================
col_comprobarColisiones:

    ld  b, #01
    ;; Case 1: Eliminate the enemy
    ;; Collision bullet hero with enemy
    ld     iy, #enemy_data
    ld     ix, #disparo_data
    call    col_checkCollision

    ld  b, #02
    ;; Case 2: The hero loses a life (in case of being the last the game is restarted)
    ;; Collision bullet enemy with hero
    call    player_getPlayerIY
    ld     ix, #disparoE_data
    call    col_checkCollision

    ld  b, #03
    ;; Case 3: The hero shoots the obstacle and this disappears
    ;; Collision bullet enemy with hero
    ld     ix, #obstacle_data
    ld     iy, #disparo_data
    call    col_checkCollision

    ld  b, #04
    ;; Case 4: The hero loses a life (in case of being the last the game is restarted) 
    ;; Collision bowl obstacle with hero
    call    player_getPlayerIY
    ld     ix, #obstacle_data
    call    col_checkCollision

    ld  b, #05
    ;; Case 5: The hero loses a life (in case of being the last the game is restarted)
    ;; Collision toast obstacle with hero
    call    player_getPlayerIY
    ld     ix, #obstacleT_data
    call    col_checkCollision

    ld  b, #06
    ;; Case 6: The hero shoots the obstacle and this disappears
    ;; Collision bullet enemy with hero
    ld     ix, #obstacleT_data
    ld     iy, #disparo_data
    call    col_checkCollision

    ld  b, #07
    ;; Case 7: The knfe suicides before the hero
    ;; Collision knife with hero
    ld     ix, #cuchillo_data
    call    player_getPlayerIY
    call    col_checkCollision

    ld  b, #08
    ;; Case 8: The hero shoots the obstacle and this disappears
    ;; Collision bullet hero with knife 
    ld     iy, #cuchillo_data
    ld     ix, #disparo_data
    call    col_checkCollision

    ret

;;====================================================
;; Performs colliion checks
;; INPUT: IX -> Entity 1 collided
;;        IY -> Entity 2 collided
;; DESTROYS: A, IY, IX
;;====================================================
col_checkCollision:
    
    ld a, de_x(iy)
    add de_w(iy)
    add #2
    sub de_x(ix)

    jp m, no_col
    jr z, no_col

    ld a, de_x(ix)
    add de_w(ix)
    sub de_x(iy)

    jp m, no_col
    jr z, no_col

    ld a, de_y(ix)
    add de_h(ix)
    sub de_y(iy)

    jp m, no_col
    jr z, no_col

    ld a, de_y(iy)
    add de_h(iy)
    sub de_y(ix)

    jp m, no_col
    jr z, no_col

    ;; Collision
    jp  col

;; No collision
no_col:
    ld a, #02
    ld (limit),a
    ld a, #0xFF

    jp check

col:
 ;;collision
    ld a, #01
    ld (limit),a
    ld  a, #0x0F

    ld      a, b
    sub     #0x01
    jr      nz, caso2

    ;; Case 1
    ld  a, de_x(iy)
    cp  #0xFF
    ret z

    call    level_enemyKilled

    call    enemy_erase

    jp check

caso2:

    sub     #0x01
    jr      nz, caso3

    ;; Case 2
ld a, #02
    ld (limit),a
    ld a, #0xFF
    call    bullet_erase
    call    menu_changeFace
    call    menu_updateLife
    call    player_changeLife

    jp      check

caso3:

    sub     #0x01
    jr      nz, caso4

    ld      iy, #obstacle_data

    ;; Case 3
    ld  a, de_x(iy)
    cp  #0xFF
    ret z

    call    level_enemyKilled

    call    bullet_getBullet
    call    bullet_erase
    call    obstacle_erase

    jp      check

caso4:

    sub     #0x01
    jr      nz, caso5

    ;; Case 4
    call    obstacle_getObstacleIY
    call    obstacle_erase

    call    menu_changeFace
    call    menu_updateLife
    call    player_changeLife

    call    level_enemyKilled

    jp      check

caso5:

    sub     #0x01
    jr      nz, caso6

    ;; Case 5
    call    obstacleT_getObstacleIY
    call    obstacle_erase

    call    menu_changeFace
    call    menu_updateLife
    call    player_changeLife 

    call    level_enemyKilled

    jp      check


caso6:

    sub     #0x01
    jr      nz, caso7

    ld      iy, #obstacleT_data

    ;; Case 6
    ld  a, de_x(iy)
    cp  #0xFF
    ret z

    call    level_enemyKilled

    call    bullet_getBullet
    call    bullet_erase
    call    obstacle_erase

caso7:

    sub     #0x01
    jr      nz, caso8

    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;; Case 7
    call    menu_changeFace
    call    menu_updateLife
    call    player_changeLife 

    call    level_enemyKilled

    ld      iy, #cuchillo_data
    call    enemy_erase

    jp      check


caso8:

    ld  a, de_x(iy)
    cp  #0xFF
    ret z

    ;; Case 8
    call    level_enemyKilled

    call    bullet_erase
    call    enemy_erase

check:
    call    level_checkChanges
    
    ret       

;;====================================================
;; Character collisions
;; INPUT: A -> Object 1
;;        B -> Limit
;; DESTROYS: A, B
;;====================================================
col_Limites:
    
    cp b
    jp m, limites   

    ;; If you have not collided with one of your limits
    ld  a, #01    ;;Output 1
    
    ret 

limites:
        
    ;; If it has collided with one of its limits   
    ld a, #02     ;;Output 2 

    ret

;;====================================================
;; Collisions in the axis X
;; INPUT: IY -> Object 1
;;        IX -> Object 2
;; DESTROYS: A
;;====================================================
col_Limites_total_X:
    

    ld a, de_x(ix)
    cp #8  ;; Limite izquierdo
    jp m, limits_X    
    cp #252 ;;LImite derecho
    jp m, limits_X 

    ;;Si ha colisionado con uno de los limites en x
    ld  a, #0x00    ;;output 1
    
    ret 

limits_X:

    ;;Si no ha colisionado con uno de los limites en x
    ld a, #0x01     ;;output 0

    ret


;;====================================================
;; Collisions in the axis Y
;; INPUT: IY -> Object 1
;;        IX -> Object 2
;; DESTROYS: A
;;====================================================
col_Limites_total_Y:
    

    ld a, de_y(ix) ;;load y in a
    cp #10    ;;upper limit (if y/2-13 < 0)
    jp m, limits_Y
    cp #-8   ;;upper limit (if y/2-13 < 0)
    jp m, limits_Y

    ;;if it doesn't limit 
    ld  a, #0x00    ;;output 1
    ret 

    ;;if it limits in any axis
limits_Y:

    ld a, #0x01     ;;output 0

    ret


;;====================================================
;; Get x (left) from the map
;; DESTROYS: B
;;====================================================
col_getIzqMap:

    ld b, #10

    ret


;;====================================================
;; Get x (right) from the map
;; DESTROYS: B
;;====================================================
col_getDerMap:
    
    ld b, #249

    ret

;;====================================================
;; Get y (up) from the map
;; DESTROYS: B
;;====================================================
col_getUpMap:

    ld b, #10

    ret

;;====================================================
;; Get y (down) from the map
;; DESTROYS: B
;;====================================================
col_getDownMap:
    
    ld b, #-8

    ret