;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH ENEMYS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "bullet.h.s"
.include "collision.h.s"
.include "obstacle.h.s"
.include "enemy.h.s"
.include "player.h.s"
.include "menu.h.s"
.include "level.h.s"

.globl _cucharaE_sp
.globl _cuchillo_sp

DefineEnemigo enemy_data, 0xFF, 0xFF, 0x00, 0xFD, 0x05, 0x10, _cucharaE_sp, enemy_moveY, 0x00, 0x00
DefineEnemigo cuchillo_data, 0xFF, 0xFF, 0x00, 0xFD, 0x05, 0x10, _cuchillo_sp, enemy_move3, 0x00, 0x00

move_cd: .db 25
move_max: .db 05
move_min: .db 00

;;=============================================
;; Initialize enemy position
;; DESTROYS: A, B, IX
;;=============================================
enemy_init:

    call	enemy_getEnemy

    ld      de_x(ix), a
    ld      de_y(ix), b

    ret


;;=============================================
;; Initialize enemy knife position
;; DESTROYS: A, B
;;=============================================
enemyC_init:

    ld      de_x(ix), a
    ld      de_y(ix), b

    ret

;;=============================================
;; Update enemy position
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A, HL
;;=============================================
enemy_update:

    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;; Update the position
    ld  h, e_up_h(ix)
    ld  l, e_up_l(ix)       
    jp  (hl)

    ret

;;=============================================
;; Draw the enemies
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A
;;=============================================
enemy_draw:
    
    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;;Draws on the screen
    jp    ren_drawEntity


;;=============================================
;; Destroy the enemy
;; INPUT: IY -> Ptr to enemy array
;; DESTROYS: A
;;=============================================
enemy_erase:

    ld  a, de_x(iy)
    cp  #0xFF
    ret z	

    ld      de_x(iy), #0xFF
    ld      de_y(iy), #0xFF

    ret

;;===============================================
;; Check for a bullet and initialize its position
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A
;;===============================================
enemy_bullet:

    call    enemy_getEnemy

    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    call  bullet_getBulletE
    ld  a, de_x(ix)
    cp  #0xFF
    ret  nz

    call    bullet_posInicialE

    ret

;;===============================================
;; Enemy pattern on the y axis
;; DESTROYS: A, IX
;;===============================================
enemy_moveY:

        call ent_move
        ;;Check if is at the limit 
        call col_Limites_total_Y
        cp #00
        ret z      ;; if is at the limit, call reset position to do the loop

        ;; Check type 0 
        ld a, e_pt(ix)
        cp #0
        jp z, type0y

            ;; Check type 1
            ld a, e_pt(ix)
            cp #1
            jp z, type1y


        type0y:
            

            ld a, de_y(ix)
            inc a
            ld de_y(ix), a 
            ld a, #4
            ld e_vy(ix), a
            ld a, #1
            ld e_pt(ix), a
            ret


        type1y:
            
            ld a, de_y(ix)
            dec a
            ld de_y(ix), a
            ld a, #-4
            ld e_vy(ix), a
            ld a, #0
            ld e_pt(ix), a
            ret

    ret

;;===============================================
;; Enemy pattern on the x axis
;; DESTROYS: A, IX
;;===============================================
enemy_moveX:

        call ent_move

        ;; Check that it is ast the limit of X
        call col_Limites_total_X
        cp #00
        ret z 

        ;; Pattern type 0
        ld a, e_pt(ix)
        cp #0
        jp z, type0x

            ;; Pattern type 1
            ld a, e_pt(ix)
            cp #1
            jp z, type1x

        type0x:  

            ld a, de_x(ix)
            inc a
            ld de_x(ix), a
            ld a, #4
            ld e_vx(ix), a
            ld a, #1
            ld e_pt(ix), a
            ret


        type1x:
            
            ld a, de_x(ix)
            dec a
            ld de_x(ix), a 
            ld a, #-4
            ld e_vx(ix), a
            ld a, #0
            ld e_pt(ix), a
            ret

    ret

;;===============================================
;; Enemy pattern in the y and x axis
;; DESTROYS: A, IX
;;===============================================
enemy_moveXY:

        call ent_move
        ;; Check that it is ast the limit of both axis 
        call col_Limites_total_X
        cp #00
        jr z, patron_Y

        ;; Pattern type 0
        ld a, e_pt(ix)
        cp #0
        jp z, type0xy

            ;; Pattern type 1
            ld a, e_pt(ix)
            cp #1
            jp z, type1xy


        type0xy:

            ld a, de_x(ix)
            inc a
            ld de_x(ix), a
            ld a, #3
            ld e_vx(ix), a
            ld a, #1
            ld e_pt(ix), a
            jr  patron_Y


        type1xy:
            
            ld a, de_x(ix)
            dec a
            ld de_x(ix), a
            ld a, #-3
            ld e_vx(ix), a
            ld a, #0
            ld e_pt(ix), a


    patron_Y:
        call col_Limites_total_Y
        cp #00
        ret z  

        ;; Pattern type 0
        ld a, e_pt2(ix)
        cp #0
        jp z, type2xy

            ;; Pattern type 1
            ld a, e_pt2(ix)
            cp #1
            jp z, type3xy

        type2xy:

            ld a, de_y(ix)
            inc a
            ld de_y(ix), a
            ld a, #4
            ld e_vy(ix), a
            ld a, #1
            ld e_pt2(ix), a

            ret

        type3xy:

            ld a, de_y(ix)
            dec a
            ld de_y(ix), a
            ld a, #-4
            ld e_vy(ix), a
            ld a, #0
            ld e_pt2(ix), a

            ret


    ret

;;===============================================
;; Enemy AI
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A, B, IY
;;===============================================
enemy_move2:

        call ent_move

        ld iy, #hero_data

        ld a, de_y(iy)
        cp de_y(ix)
        jp m, subir


       ld e_vy(ix), #2
       jp lat

       subir:
       ld e_vy(ix), #-2

       lat:
       ;Hero position less enemy
       ld a, de_x(iy)
       sub de_x(ix)
       jp m, goL       ; If less than zero, enemy to left

       ;Go right
       ;First check distance
       cp #13  ; Security distance
       jp m,alejarseiz
       jp z, parar

       ld e_vx(ix), #1
       ;There is a gap
       ret

        alejarseiz:
        call col_getIzqMap
        ld a, de_x(ix)
        cp b
        jp m,parar
        jp z,parar

        ld e_vx(ix),#-1
        ret
        

       goL:

       cp #-13
       jp m,izq ;Now I must make the right enemy go to the left and keep distance
       jp z,parar

        ;Move away to the right
        call col_getDerMap
        ld a, de_x(ix)
        cp b
        jp m, parar
        jp z, parar

        ld e_vx(ix),#1
        ret

        izq:

       ld e_vx(ix), #-1
       ;There is a gap
       ret
    parar:
        ld e_vx(ix),#0
        ret


;;===============================================
;; Suicide pattern
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A, IY
;;===============================================
enemy_move3:

        call ent_move
        ;;Check if is at the limit 
        ld iy, #hero_data

        ld a, de_y(iy)
        cp de_y(ix)
        jp m, goUp

       ;Go down

       ld e_vy(ix), #1
       jp lad

       goUp:
       ld e_vy(ix), #-1

       lad:
       ;Hero position less enemy
       ld a, de_x(iy)
       cp de_x(ix)
       jp m, izqu    ;If less than zero, enemy to right
      

       ld e_vx(ix), #1
       ;There is a gap
       ret

        izqu:

       ld e_vx(ix), #-1
       ;There is a gap
       ret


;;===============================================
;; Enemy shoot
;; INPUT: IX -> Ptr to enemy array
;; DESTROYS: A, IY, IX
;;===============================================
enemy_checkShoot:
    ld iy, #hero_data

    ld a, de_y(iy)
    add #3
    sub de_y(ix)
    jr z, ejecutar
    sub #1
    jr z, ejecutar
    sub #1
    jr z, ejecutar
    sub #1
    jr z, ejecutar
    sub #1
    ret nz

ejecutar:

    ld a, de_x(iy)
    sub de_x(ix)
    jp m, hero_izq

       
    cp #15  ;Security distance
    jp m,shootD
    ret

    shootD:
    ld a, #3   ;Shoot to the right
    ld ix, #disparoE_data
    ld e_vx(ix), a
    ld a , de_x(ix)
    add de_w(iy)

    jp final

    hero_izq:

    cp #-15  ;Security distance
    ret m
    

    shootI:
    ld a, #-3   ;Shoot to the left
    ld ix, #disparoE_data
    ld e_vx(ix), a

    final: 
    
    call enemy_bullet

    ret

;;===============================================
;; Get the enemy
;; DESTROYS: IX
;;===============================================
enemy_getEnemy:

    ld  ix, #enemy_data

    ret