;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH ENTITYS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"

.globl _disparo_sp
.globl _disparoE_sp
.globl _bol_sp
.globl _tostada_sp

k_max_num_ent = 10
k_entity_size = entity_vector0_size

entity_vector:    
DefineNEntities entity_vector, k_max_num_ent
DefineEntity disparo_data, 0xFF, 0xFF, 0x00, 0x00, 0x04, 0x08, _disparo_sp, ent_move
DefineEntity disparoE_data, 0xFF, 0xFF, 0xFE, 0x00, 0x04, 0x08, _disparoE_sp, ent_move


m_num_ent:      .db 00
m_next_entity:  .dw entity_vector


rand: .db #01

;;=============================================
;; Register a new entity
;; DESTROYS: AF, HL, BC
;; OUTPUT: HL -> Ptr to entity
;;=============================================
ent_new:

    ld      a, (m_num_ent)
    inc     a
    ld      (m_num_ent), a

    ld      hl, (m_next_entity)
    ld      bc, #k_entity_size
    add     hl, bc
    ld      (m_next_entity), hl
    or      a   
    sbc     hl, bc

    ret

;;=============================================
;; Copies the values of one entity over another
;; DESTROYS: BC
;; INPUT: HL -> Origin entity
;;        DE -> Destination entity
;;=============================================
ent_copy:

    ld      bc, #k_entity_size
    ldir
    ret

;;=============================================
;; Draw all the entitys
;; DESTROYS: AF, IX, BC,
;; INPUT: HL -> Ptr to the method to execute
;;=============================================

ent_doForAll:
    ld      a, (m_num_ent)
    ld     ix, #entity_vector
    ld     (metodo), hl

buc:
    push   af
    metodo = . + 1
    call    ent_draw
    pop    af
    ld     bc, #k_entity_size
    add    ix, bc

    dec     a
    jr     nz, buc

    ret

;;=============================================
;; Draw a moving entity
;; DESTROYS: AF, BC, DE, HL
;; INPUT: HL -> Ptr to the entity
;;=============================================
ent_draw:
    jp    ren_drawEntity


;;=============================================
;; Draw a fixed entity
;; DESTROYS: AF, BC, DE, HL
;; INPUT: HL -> Ptr to the entity
;;=============================================
ent_drawFixed:
    jp    ren_drawEntityFixed


;;=============================================
;; Update an entity
;; DESTROYS: HL
;; INPUT: IX -> Ptr to the entity
;;=============================================
ent_update:
    ld 	h, e_up_h(ix)
    ld 	l, e_up_l(ix)		
    jp	(hl)


;;=============================================
;; Move an entity
;; DESTROYS: A
;; INPUT: IX -> Ptr to the entity
;;=============================================
ent_move:
    ld      a, de_x(ix)
    add     e_vx(ix)
    ld      de_x(ix), a

    ld      a, de_y(ix)
    add     e_vy(ix)
    ld      de_y(ix), a

    ret