;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH LEVELS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "player.h.s"
.include "obstacle.h.s"
.include "bullet.h.s"
.include "menu.h.s"
.include "level.h.s"
.include "map.h.s"

CurrentLevel: .db  00
numEnemys: 	  .db  00

.globl _caraFeliz_sp
.globl _cuchara_sp_0

;;=============================================
;; Go to the next level
;; DESTROYS: A, BC, IX, HL
;;=============================================
level_changeLevel:

	 ;;Collision
    	 ld	a, (CurrentLevel)
    	 cp   #00
    	 jr   nz, level_1

    	 	;; Load the first level
    	 	ld	b, #02
    	 	call	level_setEnemys

    	 	;; Reset camera position
    	 	call	map_initCoordX

    	 	;; Initialize hero
    	 	ld      a, #0x28
    		ld      b, #0x40
    	 	call	player_init

    	 	ld	  c, #0x01
    	 	call	player_initLife

    	 	;;Initialize enemy
    	 	;ld      a, #0x8
    		;ld      b, #0x50
    	 	;call  enemy_init

    	 	;; Initialize bowl obstacle
    	 	ld      a, #0x50
    		ld      b, #0x15
    	 	call  obstacle_init

    	 	;; Initialize toast obstacle
    	 	ld	  a, #0x40
    	 	ld	  b, #0x65
    	 	call	obstacleT_init

    	 	;; Initialize hero's bullet
    	 	;call	bullet_init

    	 	;; Save the next level
    	 	call	level_setLevel

    	 	ret

level_1:

    	 cp   #01
    	 jr   nz, level_2	

		    ;; Load the second level
    	 	ld	b, #01
    	 	call	level_setEnemys

            ;; Reset camera position
    	 	call	map_initCoordX

            ;; Initialize hero
    	 	ld      a, #0x28
    		ld      b, #0x40
    	 	call	player_init

            ;;Initialize enemy
            ld      ix, #cuchillo_data
    	 	ld      a, #0x80
    		ld      b, #0x50
    	 	call  enemyC_init

    	 	;; Enemy pattern
    	 	;ld hl, #enemy_moveY
    		;ld  e_up_l(ix), l
    		;ld  e_up_h(ix), h

            ;; Initialize bowl obstacle
    	 	;ld      a, #0x20
    		;ld      b, #0x15
    	 	;call  obstacle_init

            ;; Initialize toast obstacle
    	 	;ld	  a, #0x20
    	 	;ld	  b, #0x65
    	 	;call	obstacleT_init

            ;; Initialize hero's bullet
    	 	;call	bullet_init

            ;; Save the next level
    	 	call	level_setLevel

    	 	ret

level_2:

    	 cp   #02
    	 jr   nz, level_3	

		    ;; Load the third level
    	 	ld	b, #03
    	 	call	level_setEnemys

            ;; Reset camera position
    	 	call	map_initCoordX

            ;; Initialize hero
    	 	ld      a, #0x28
    		ld      b, #0x40
    	 	call	player_init

            ;;Initialize enemy
    	 	ld      a, #0x80
    		ld      b, #0x60
    	 	call  enemy_init

            ;; Enemy pattern
    	 	ld hl, #enemy_move2
    		ld  e_up_l(ix), l
    		ld  e_up_h(ix), h

            ;; Initialize bowl obstacle
    	 	ld      a, #0x40
    		ld      b, #0x15
    	 	call  obstacle_init

            ;; Initialize toast obstacle
    	 	ld	  a, #0x50
    	 	ld	  b, #0x30
    	 	call	obstacleT_init

            ;; Initialize hero's bullet
    	 	;call	bullet_init

            ;; Save the next level
    	 	call	level_setLevel

    	 	ret


level_3:

    	 cp   #03
    	 jr   nz, level_4

            ;; Load the fourth level
    	 	ld	b, #01
    	 	call	level_setEnemys

            ;; Reset camera position
    	 	call	map_initCoordX

            ;; Initialize hero
    	 	ld      a, #0x28
    		ld      b, #0x40
    	 	call	player_init

            ;;Initialize enemy
    	 	ld      a, #0x80
    		ld      b, #0x50
    	 	call  enemy_init

            ;; Initialize bowl obstacle
    	 	;ld      a, #0x20
    		;ld      b, #0x15
    	 	;call  obstacle_init

            ;; Initialize toast obstacle
    	 	;ld	  a, #0x20
    	 	;ld	  b, #0x65
    	 	;call	obstacleT_init

            ;; Initialize hero's bullet
    	 	;call	bullet_init

            ;; Save the next level
    	 	call	level_setLevel

    	 	ret

level_4:

    	 cp   #04
    	 jr   nz, level_5

            ;; Load the bonus level
    	 	ld	b,  #03
    	 	call	level_setEnemys

            ;; Reset camera position
    	 	call	map_initCoordX

            ;; Initialize hero
    	 	ld      a, #0x28
    		ld      b, #0x40
    	 	call	player_init

    	 	ld	  c, #0x01
    	 	call	player_initLife

            ;;Initialize enemy
    	 	ld      a, #0x50
    		ld      b, #0x20
    	 	call  enemy_init

            ;; Initialize bowl obstacle
    	 	ld      a, #0x60
    		ld      b, #0x25
    	 	call  obstacle_init

            ;; Initialize toast obstacle
    	 	ld	  a, #0x60
    	 	ld	  b, #0x50
    	 	call	obstacleT_init

            ;; Initialize hero's bullet
    	 	;call	bullet_init

            ;; Save the next level
    	 	call	level_setLevel

    	 	ret

level_5:

	call level_reset_game

	ret


;;=============================================
;; Restart the game
;; DESTROYS: A, IX, IY, HL
;;=============================================
level_reset_game:

    ;; Reset bullet of the hero
    call    bullet_getBullet
    call    bullet_erase

    ;; Reset bullet of the enemy
    call    bullet_getBulletE
    call    bullet_erase

    ;; Stop the music
    call    cpct_akp_stop_asm

    call	map_initCoordX

    ;; We return to level 0
    ld	a, #00
    ld	(CurrentLevel), a

    ;; Reset number of enemies
    ld	a, #00
    ld	(numEnemys), a

    ;; We return to the start menu
    call    menu_system

    ;; Restart face animation
    ld ix, #hud_data_cara
    ld hl, #_caraFeliz_sp
    ld  de_sp_l(ix), l
    ld  de_sp_h(ix), h

    ;; Restart animation lives
    ld ix, #hud_data_vida2
    ld  de_x(ix), #0x37
    ld  de_y(ix), #0x99

    ;; Restart character animation
    call	 player_getPlayer
    ld hl, #_cuchara_sp_0
    ld  de_sp_l(ix), l
    ld  de_sp_h(ix), h

    ;; We finish with all
    ld     iy, #enemy_data
    call	enemy_erase

    ld     iy, #cuchillo_data
    call    enemy_erase

    call 	obstacle_getObstacleIY
    call	obstacle_erase

    call 	obstacleT_getObstacleIY
    call	obstacle_erase

    call	level_changeLevel

    ret


;;====================================================
;; CHeck if you need to pass level or restart the game
;; DESTROYS: A, IY
;;====================================================
level_checkChanges:

	;; First we check that the player has some lives left

	call	  player_getPlayerIY	
	ld  	  a, e_hp(iy)
      cp      #0xFF
      jp      z, reset

	ld	a, (numEnemys)
	;; If there are no more enemies, it restarts
    	cp   #00
    	jp   z, level_changeLevel

    	ret

reset:

	call	level_reset_game

	ret


;==============================================
;; Increase the variable for level change
;; DESTROYS: A
;;=============================================
level_setLevel:

	ld	a, (CurrentLevel)
	inc	a
	ld	(CurrentLevel), a

	ret

;;=============================================
;; Change the variable for enemies in level
;; DESTROYS: A
;; INPUT: B -> Number of enemies
;;=============================================
level_setEnemys:

	ld	a, (numEnemys)
	add	b
	ld	(numEnemys), a

	ret

;;=============================================
;; Decreases in one the number of enemies
;; DESTROYS: A
;;=============================================
level_enemyKilled:

	ld	a, (numEnemys)
	dec	a
	ld	(numEnemys), a

	ret