;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; MAIN FILE
;;==============================================================

.area _DATA
.area _CODE

;;==============================================================
;; INCLUDE AREA
;;==============================================================
.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "bullet.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "player.h.s"  
.include "menu.h.s"
.include "obstacle.h.s"
.include "map.h.s"
.include "level.h.s"


.globl _cuchara_pal
.globl _palette2


_main::
    ld      sp, #0x8000

    ;; Disable firmware to prevent it from interfering with string drawing
    call    cpct_disableFirmware_asm

    ;; Initialize video mode in 0
    ld      c, #0
    call    cpct_setVideoMode_asm

    ;; Initialize palette
    ld      hl, #_cuchara_pal
    ld      de, #16
    call    cpct_setPalette_asm

    ;; Initialize the edge palette
    ld      hl, #0x1416 
    call    cpct_setPALColour_asm

    call    ren_initBuffers

    ;; Draw the principal menu
    call    menu_system

    ;; Load first level
    call    level_changeLevel

    ;; Initialize the map
    call    map_init

loop: 

    ;; Draw and update the map
    call    map_draw


    ld a, (CurrentLevel)
    cp #01
    jp nz,leap
    ld      ix, #guinyo_data
    call    ent_draw
    leap:

    ;; Update position and drawing of the bowl obstacle
    ld     ix, #obstacle_data
    call    obstacle_update
    call    obstacle_draw

    ;; Update position and drawing of the toast obstacle
    ld     ix, #obstacleT_data
    call    obstacle_update
    call    obstacle_draw

    ;; Update position and drawing of the hero
    ld     ix, #hero_data
    call    ent_update
    call    ent_draw

    ;; Update position and drawing of the enemy
    ld     ix, #enemy_data
    call    enemy_update
    call    enemy_draw
    call    enemy_checkShoot

    ;; Update position and drawing of the knife
    ld     ix, #cuchillo_data
    call    enemy_update
    call    enemy_draw

    ;; Hero bullet
    call    bullet_getBullet
    call    bullet_update
    call    bullet_draw
    call    bullet_limits

    ;; Enemy bullet
    call    bullet_getBulletE
    call    bullet_update
    call    bullet_draw
    call    bullet_limits

    ;; Check all the collisions
    call    col_comprobarColisiones

    ;; Draw the HUD
    call    draw_hud

    
    ;; Pressing the Space key creates a new bullet for the hero
    ld	hl, #Key_Space
    call	cpct_isKeyPressed_asm
    jr	z, n_no_pulsada
        ;; We check if the bullet is created or not
        
        call    bullet_getBullet
        call    bullet_check


n_no_pulsada:

    ;; Double buffer
    call    cpct_waitVSYNC_asm
    call    ren_newScene   

    ;; Loop forever
    jp    loop
