;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH MAPS
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "map.h.s"
.include "main.h.s"
.include "render.h.s"
.include "player.h.s"
.include "level.h.s"
.include "menu.h.s"

.globl _levels_pack_end

decompress_buffer       = 0x040
levelmaxsize            = 0xDD0
decompress_buffer_end   = decompress_buffer + levelmaxsize - 1
tileset_ptr             = decompress_buffer + 2880
tilemap_ptr             = decompress_buffer + 0
tilemap1_ptr            = decompress_buffer + 576
tilemap2_ptr            = decompress_buffer + 1152
tilemap3_ptr            = decompress_buffer + 1728
tilemapBonus_ptr        = decompress_buffer + 2304

;;=============================================
;; Inicializa el mapa
;; REGISTROS DESTRUIDOS:
;;=============================================

coord_x: .db 0x00


;;=============================================
;; Initialize map position
;; DESTROYS: HL, DE, BC
;;=============================================
map_init:

    ld       hl, #_levels_pack_end
    ld       de, #decompress_buffer_end
    call     cpct_zx7b_decrunch_s_asm

    ld       bc, #0x01212        ;; Viewport 18x18 tiles
    ld       de, #32             ;; Tilemap Width
    ld       hl, #tileset_ptr    ;; Tileset pointer

    call cpct_etm_setDrawTilemap4x8_ag_asm

    ret

;;=============================================
;; Update map position when you press D
;; DESTROYS: A, HL, DE
;;=============================================
map_updateD:

        ld  a, (coord_x)
        cp  #56
        ret z

        ld   a, (CurrentLevel)
        cp   #00
        jr   nz, pasaD1

        ld  hl, (tilemap)
        ld  de, #1
        add hl, de
        ld  (tilemap), hl

        jp  incrementosD

        pasaD1:

        cp   #01
        jr   nz, pasaD2

        ld  hl, (tilemap1)
        ld  de, #1
        add hl, de
        ld  (tilemap1), hl

        jp  incrementosD

        pasaD2:

        cp   #02
        jr   nz, pasaD3

        ld  hl, (tilemap2)
        ld  de, #1
        add hl, de
        ld  (tilemap2), hl

        jp  incrementosD

        pasaD3:

        cp   #03
        jr   nz, pasaD4

        ld  hl, (tilemap3)
        ld  de, #1
        add hl, de
        ld  (tilemap3), hl

        pasaD4:

        ld  hl, (tilemapBonus)
        ld  de, #1
        add hl, de
        ld  (tilemapBonus), hl

        incrementosD:
        ;; Increase its value on the x axis by 4
        ld  a, (coord_x)
        inc a
        inc a
        inc a
        inc a
        ld  (coord_x), a

        ret

;;=============================================
;; Update map position when you press A
;; DESTROYS: A, HL, DE
;;=============================================
map_updateA:

       ld  a, (coord_x)
       cp  #0
       ret z

       ld   a, (CurrentLevel)
       cp   #00
       jr   nz, pasaA1

       ld  hl, (tilemap)
       ld  de, #-1
       add hl, de
       ld  (tilemap), hl

       jp  incrementosA

       pasaA1:

       cp   #01
       jr   nz, pasaA2

       ld  hl, (tilemap1)
       ld  de, #-1
       add hl, de
       ld  (tilemap1), hl

       jp  incrementosA

       pasaA2:

       cp   #02
       jr   nz, pasaA3

       ld  hl, (tilemap2)
       ld  de, #-1
       add hl, de
       ld  (tilemap2), hl

       jp  incrementosA

       pasaA3:

       cp   #03
       jr   nz, pasaA4

       ld  hl, (tilemap3)
       ld  de, #-1
       add hl, de
       ld  (tilemap3), hl

       pasaA4:

       ld  hl, (tilemapBonus)
       ld  de, #-1
       add hl, de
       ld  (tilemapBonus), hl

       incrementosA:

       ;; Decrease its value on the x axis by 4
       ld  a, (coord_x)
       dec a
       dec a
       dec a
       dec a
       ld  (coord_x), a

    ret

;;=============================================
;; Draw the map
;; DESTROYS: A, HL, DE
;;=============================================
map_draw:

    ld      a, (m_back_buffer)
    ld      h, a
    ld      l, #04    ;; 20 tiles per row (80 bytes) and we have 18x18 viewport
                      ;; so we have to leave 1 separation on each side

    ld   a, (CurrentLevel)
    cp   #00
    jr   nz, level_1

    tilemap = . + 1
    ld   de, #tilemap_ptr     ;; First tile that we have to draw
    jp   llamada 

level_1:

    cp   #01
    jr   nz, level_2

    tilemap1 = . + 1
    ld   de, #tilemap1_ptr    ;; First tile that we have to draw

    jp   llamada

level_2:

    cp   #02
    jr   nz, level_3

    tilemap2 = . + 1
    ld   de, #tilemap2_ptr    ;; First tile that we have to draw

    jp   llamada

level_3:
    cp   #03
    jr   nz, level_4

    tilemap3 = . + 1
    ld   de, #tilemap3_ptr    ;; First tile that we have to draw

    jp   llamada

level_4:
    tilemapBonus = . + 1
    ld   de, #tilemapBonus_ptr    ;; First tile that we have to draw

llamada:
    call     cpct_etm_drawTilemap4x8_ag_asm 

    ret


;;==============================================
;; Check if you move the camera when you press A
;; DESTROYS: A, IX
;;==============================================
map_checkMoveA:

    call player_getPlayer
    ld  a, de_x(ix)
    cp  #0x28
    jp  m, no_hacerloA

    ;; The movement of the camera is not done
    ld  a, #0x00

    ret

no_hacerloA:

    ;; The movement of the camera is done
    ld  a, #0x01

    ret


;;==============================================
;; Check if you move the camera when you press D
;; DESTROYS: A, IX
;;==============================================
map_checkMoveD:

    call player_getPlayer
    ld  a, de_x(ix)
    cp  #0xE0
    jp  m, no_hacerloD

    ;; The movement of the camera is not done
    ld  a, #0x00

    ret

no_hacerloD:

    ;; The movement of the camera is done
    ld  a, #0x01

    ret


;;==============================================
;; Reset the camera position after each level
;; DESTROYS: AF, HL, DE
;;==============================================
map_initCoordX:

    ;; Si no 
    ld    a, (coord_x)
    cp    #0x00
    ret   z

    push  af

    ld   a, (CurrentLevel)
    cp   #00
    jp   nz, init1

         pop af

         bucle:

         ld  hl, (tilemap)
         ld  de, #-1
         add hl, de
         ld  (tilemap), hl

         sub  #0x04
         jp  nz, bucle

         jp end

init1:

  cp   #01
  jp   nz, init2

  pop af

  bucle1:

         ld  hl, (tilemap1)
         ld  de, #-1
         add hl, de
         ld  (tilemap1), hl

         sub  #0x04
         jp  nz, bucle1

         jp end

init2:

  cp   #02
  jp   nz, init3

  pop af

  bucle2:

         ld  hl, (tilemap2)
         ld  de, #-1
         add hl, de
         ld  (tilemap2), hl

         sub  #0x04
         jp  nz, bucle2

         jp end

init3:

  cp   #03
  jp   nz, init4

  pop af

  bucle3:

         ld  hl, (tilemap3)
         ld  de, #-1
         add hl, de
         ld  (tilemap3), hl

         sub  #0x04
         jp  nz, bucle3

         jp end

init4:

  cp   #04
  jp   nz, end

  pop af

  bucle4:

         ld  hl, (tilemapBonus)
         ld  de, #-1
         add hl, de
         ld  (tilemapBonus), hl

         sub  #0x04
         jp  nz, bucle4

end:

    ld    a, #0x00
    ld    (coord_x), a

    ret

;;==============================================
;; Get the x coordinate
;; DESTROYS: A
;; OUTPUT: C -> Coordinate X
;;==============================================
map_getCoordX:

    ld    a, (coord_x)
    ld    c, a

    ret

