;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH OBSTACLES
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "obstacle.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "level.h.s"

.globl _bol_sp
.globl _tostada_sp

DefineEnemigo obstacle_data, 0xFF, 0xFF, 0x03, 0x04, 0x04, 0x08, _bol_sp, enemy_moveXY, 0x01, 0x01
DefineEnemigo obstacleT_data, 0xFF, 0xFF, 0x00, 0xFC, 0x04, 0x08, _tostada_sp, enemy_moveY, 0x00, 0x00


;;=============================================
;; Update the bowl obstacle position
;; INPUT: IX -> Ptr to bowl obstacle array
;; DESTROYS: A, HL
;;=============================================
obstacle_update:

	   ld  a, de_x(ix)
     cp  #0xFF
     ret z

	   ld  h, e_up_h(ix)
	   ld  l, e_up_l(ix)       
	   jp  (hl)

	ret

;;=============================================
;; Update the toast obstacle position
;; INPUT: IX -> Ptr to toast obstacle array
;; DESTROYS: A, HL
;;=============================================
obstacleT_update:

  ld  a, de_x(ix)
      cp  #0xFF
      ret z

      ld  h, e_up_h(ix)
      ld  l, e_up_l(ix)       
      jp  (hl)

  ret


;;=============================================
;; Draw the obstacle
;; INPUT: IX -> Ptr to obstacle array
;; DESTROYS: A
;;=============================================
obstacle_draw:

    ld  a, de_x(ix)
    cp  #0xFF
    ret z

    ;;Lo dibujad por pantalla
    jp    ren_drawEntity


;;=============================================
;; Initialize the bowl obstacle position
;; INPUT: A -> Position in X
;;        B -> Position in Y
;; DESTROYS: IX
;;=============================================
obstacle_init:

	call    obstacle_getObstacle

  ld      de_x(ix), a
  ld      de_y(ix), b

	ret

;;=============================================
;; Initialize the toast obstacle position
;; INPUT: A -> Position in X
;;        B -> Position in Y
;; DESTROYS: IX
;;=============================================
obstacleT_init:

  ld      ix, #obstacleT_data

  ld      de_x(ix), a
  ld      de_y(ix), b

  ret

;;=============================================
;; Erase the obstacle
;; DESTROYS: IY
;;=============================================
obstacle_erase:

  ld  a, de_x(iy)
  cp  #0xFF
  ret z

	ld      de_x(iy), #0xFF
  ld      de_y(iy), #0xFF

	ret

;;=============================================
;; Get the bowl obstacle in IX
;; DESTROYS: IX
;;=============================================
obstacle_getObstacle:

	ld	ix, #obstacle_data

	ret

;;=============================================
;; Get the bowl obstacle in IY
;; DESTROYS: IY
;;=============================================
obstacle_getObstacleIY:

  ld  iy, #obstacle_data

  ret

;;=============================================
;; Get the toast obstacle
;; DESTROYS: IY
;;=============================================
obstacleT_getObstacleIY:

  ld  iy, #obstacleT_data

  ret