;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH PLAYER
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "cpctelera.h.s"
.include "entity.h.s"
.include "main.h.s"
.include "player.h.s"
.include "bullet.h.s"
.include "collision.h.s"
.include "enemy.h.s"
.include "obstacle.h.s"
.include "map.h.s"
.include "level.h.s"

.globl _cuchara_sp_0
.globl _cuchara_sp_1
.globl _cuchara_sp_2
.globl _cuchara_sp_3
.globl hero_width

DefineHero hero_data, 0xFF, 0xFF, 0x00, 0x00, 0x05, 0x10, _cuchara_sp_0, ent_moveKeyboard, 0xFF

vel_init:       .db -8

;;==============================================================
;; Initialize player position
;; INPUT: A -> Position in X
;;        B -> Position in Y
;; DESTROYS: IX
;;==============================================================
player_init:

    call    player_getPlayer

    ld      de_x(ix), a
    ld      de_y(ix), b

    ret

;;=============================================
;; Initialize the life of the hero
;; INPUT: C -> NUmber of lives of the player
;;=============================================
player_initLife:

    ld      e_hp(ix), c

    ret


;;=============================================
;; Move player
;; INPUT: IX -> Ptr to player array
;; DESTROYS: A, HL, IY
;;=============================================
ent_moveKeyboard:
    call	cpct_scanKeyboard_asm

    ld iy, #disparo_data

    ld      a, (vel_init)
    ld      e_vx(iy), a

    ld	hl, #Key_O
    call	cpct_isKeyPressed_asm
    jr	z, a_no_pulsada

            ld  hl, #Key_P
            call    cpct_isKeyPressed_asm
            jr  nz, a_no_pulsada

        call map_checkMoveD
        cp  #0x01
        jr  z, no_updateA

        call map_updateA

        no_updateA:

        call col_getIzqMap
        ld a,de_x(ix)
        call col_Limites;;comprobar boundries
        sub #1          ;; Si da 0 esta en el limite y si da 1 sigue moviendose
        jr nz, a_no_pulsada

    	ld	e_vx(ix), #-4

        ld hl, #_cuchara_sp_0
        ld  de_sp_l(ix), l
        ld  de_sp_h(ix), h

        ld  a, de_x(iy)
        cp  #0xFF
        jr     nz, a_no_pulsada
            ld      a, #-6
            ld      (vel_init), a
            ld      e_vx(iy), a

            ld      a, #0x00
            ld      (hero_width), a

a_no_pulsada:

    ld	hl, #Key_P
    call	cpct_isKeyPressed_asm
    jr	z, d_no_pulsada

            ld  hl, #Key_O
            call    cpct_isKeyPressed_asm
            jr  nz, d_no_pulsada

        call map_checkMoveA
        cp  #0x01
        jr  z, no_updateD

        call map_updateD

        no_updateD:
        
        ld a,de_x(ix)
        call col_getDerMap
        
        call col_Limites;;comprobar boundries
        sub #1          ;; Si da 0 esta en el limite y si da 1 sigue moviendose
        jr nz, d_no_pulsada
    	ld	e_vx(ix), #4

        ld hl, #_cuchara_sp_1
        ld  de_sp_l(ix), l
        ld  de_sp_h(ix), h

        ld  a, de_x(iy)
        cp  #0xFF
        jr     nz, d_no_pulsada
            ld      a, #6
            ld      (vel_init), a
            ld      e_vx(iy), a

            ld      a, #0x01
            ld     (hero_width), a

d_no_pulsada:

    ld  hl, #Key_Q
    call    cpct_isKeyPressed_asm
    jr  z, w_no_pulsada

        ld a,de_y(ix)

        call col_getUpMap
        call col_Limites;;comprobar boundries
        sub #1          ;; Si da 0 esta en el limite y si da 1 sigue moviendose
        jr nz, w_no_pulsada

        ld  e_vy(ix), #-8

        ld hl, #_cuchara_sp_2
        ld  de_sp_l(ix), l
        ld  de_sp_h(ix), h

w_no_pulsada:

    

    ld  hl, #Key_A
    call    cpct_isKeyPressed_asm
    jr  z, s_no_pulsada

        ld a,de_y(ix)

        call col_getDownMap
        call col_Limites;;comprobar boundries
        sub #1          ;; Si da 0 esta en el limite y si da 1 sigue moviendose
        jr nz, s_no_pulsada

        ld  e_vy(ix), #8

        ld hl, #_cuchara_sp_3
        ld  de_sp_l(ix), l
        ld  de_sp_h(ix), h

s_no_pulsada:

    call ent_move

    ld  e_vx(ix), #0
    ld  e_vy(ix), #0

    ret

;;=============================================
;; Change the player's life
;; DESTROYS: A, IY
;;=============================================
player_changeLife:

      call  player_getPlayerIY
      	;; Reduce life in one
	      ld      a, e_hp(iy)
	      sub	  #0x01
	      ld      e_hp(iy), a

	ret

;;=============================================
;; Get the player in IX
;; DESTROYS: IX
;; OUTPUT: IX -> Ptr to player array
;;=============================================
player_getPlayer:

    ld ix, #hero_data

    ret

;;=============================================
;; Get the player in IY
;; DESTROYS: IY
;; OUTPUT: IY -> Ptr to player array
;;=============================================
player_getPlayerIY:

    ld iy, #hero_data

    ret