;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of Amstrad CPC videogame "Cuchara Voladora"
;;  Copyright (C) 2018 Blue Hornet / Sergio Perez Perez / Borja Garcia Pizarro 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

;;==============================================================
;; FUNCTIONS RELATED WITH RENDER
;;==============================================================

;;==============================================================
;; INCLUDE AREA
;;==============================================================

.include "render.h.s"
.include "main.h.s"
.include "map.h.s"

m_front_buffer: .db 0xC0
m_back_buffer:  .db 0x80
interruptionCount: .db 0x00


;;=============================================
;; Clean and prepare a new buffer
;;=============================================
ren_newScene:
    call    ren_switchBuffers
    call    ren_clearBackBuffer
    ret

;;=============================================
;; Exchange the buffers
;; DESTROYS: AF, BC, DE, HL
;;=============================================
ren_clearBackBuffer:
    ld      a, (m_back_buffer)
    ld      h, a
    ld     de, #0
    ld      l, e
    ld     bc, #0x4000

    call    cpct_memset_f64_asm

    ret


;;=============================================
;; Full of zeros secondary buffer
;; DESTROYS: AF, BC, DE, HL
;;=============================================
ren_initBuffers:

    ld      hl, #0x8000
    ld    (hl), #0
    ld      de, #0x8000 + 1
    ld      bc, #0x4000 - 1
    ldir

    ret

;;=============================================
;; Exchange the buffers
;; Destroys: AF, BC, DE, HL
;;=============================================
ren_switchBuffers:
    ld      a, (m_back_buffer)
    ld      b, a
    ld      a, (m_front_buffer)
    ld      (m_back_buffer), a
    ld      a, b
    ld      (m_front_buffer), a

    srl     b
    srl     b
    ld      l, b
    jp      cpct_setVideoMemoryPage_asm

;;=============================================
;; Draw an entity in movement
;; DESTROYS: AF, BC, DE, HL
;; INPUT: IX -> Ptr to entity
;;=============================================
ren_drawEntity:

    ;; Return X coordinate in C
    call    map_getCoordX
    ld      a, de_x(ix)
    cp      c
    ret     m

    ld      a, #0x45
    add     a, c
    cp      de_x(ix)
    ret     m 

    ;; We save the start of the screen pointer in DE
    ld      a, (m_back_buffer)
    ld      d, a
    ld      e, #0

    ;; Keep the X coordinate
    ld      a, de_x(ix)        ;; C = Entity Y
    sub     a, c
    ld      c, a  

    ld      b, de_y(ix)
    call    cpct_getScreenPtr_asm

    ex     de, hl           ;; DE = Ptr to memory
    ld      l, de_sp_l(ix)
    ld      h, de_sp_h(ix)
    ld      b, de_h(ix)        ;; Height    
    ld      c, de_w(ix)        ;; Width

    call    cpct_drawSpriteMasked_asm

    ret

;;=============================================
;; Draw a fixed entity
;; DESTROYS: AF, BC, DE, HL
;; INPUT: IX -> Ptr to entity
;;=============================================
ren_drawEntityFixed:

    ld      a, (m_back_buffer)
    ld      d, a
    ld      e, #0
    ld      c, de_x(ix)        ;; C = Entity Y
    ld      b, de_y(ix)        ;; B = Entity X
    call    cpct_getScreenPtr_asm

    ex     de, hl           ;; DE = Ptr to memory
    ld      l, de_sp_l(ix)
    ld      h, de_sp_h(ix)
    ld      b, de_h(ix)        ;; Height    
    ld      c, de_w(ix)        ;; Width

    call    cpct_drawSprite_asm

    ret